/// <reference types="node" />
import { MarkOptional, DeepRequired } from 'ts-essentials';
import EventEmitter from 'events';
import { TransactionRequest } from '@ethersproject/abstract-provider';
import { EventFilter, BaseContract, BigNumber, ContractTransaction, providers, Signer } from 'ethers';
import { ERC20Tornado, ETHTornado, TornadoInstance, TornadoProxy, ERC20, RelayerRegistry } from './deth';
import { Multicall3 } from './deth/Multicall3Contract';
import { Cache, Docs, Options as DataOptions } from '@tornado/sdk-data';
export type TornadoContracts = TornadoInstance | TornadoProxy | ETHTornado | ERC20Tornado | ERC20 | RelayerRegistry;
export declare namespace Options {
    interface Sync {
        startBlock?: number;
        targetBlock?: number;
        blockDelta?: number;
        blockDivisor?: number;
        concurrencyLimit?: number;
        msTimeout?: number;
        listenForEvents?: boolean;
    }
    type Cache = DataOptions.Cache;
}
type Provider = providers.Provider;
/**
 * The Chain class stores Tornado-agnostic chain data and also
 * handles such interactions.
 */
export declare class Chain {
    provider: Provider;
    private _emptySigner;
    private _signer?;
    private _chainId?;
    private _symbol?;
    private _fetched;
    constructor(provider: Provider, signer?: Signer);
    connectSigner(signer: Signer): void;
    fetchChainData(): Promise<void>;
    private _signerConnected;
    private _propertiesFetched;
    get signer(): Signer;
    get id(): number;
    get symbol(): string;
    latestBlockNum(): Promise<number>;
    getAccountBalance(account: string): Promise<BigNumber>;
    getGasPrice(): Promise<BigNumber>;
    getTokenContract(tokenAddress: string): ERC20;
    getTokenDecimals(token: string): Promise<BigNumber>;
    getTokenBalance(account: string, token: string, normalized?: boolean): Promise<BigNumber>;
    populateBatchCall(callStruct: Array<MarkOptional<Multicall3.Call3ValueStruct, 'value'>>): Promise<TransactionRequest>;
    batchCall(callStruct: Array<MarkOptional<Multicall3.Call3ValueStruct, 'value'>>): Promise<ContractTransaction>;
}
/**
 * This is Tornado-specific.
 */
export declare namespace Contracts {
    function getProxy(network: string, signerOrProvider: Signer | Provider): TornadoProxy;
    function getRegistry(signerOrProvider: Signer | Provider, network?: string): RelayerRegistry;
    function getInstance(network: string, token: string, denomination: string, signerOrProvider: Signer | Provider): TornadoInstance;
    function getToken(tokenAddress: string, signerOrProvider: Signer | Provider): ERC20;
    function getTornToken(signerOrProvider: Signer | Provider): ERC20;
}
export declare abstract class Synchronizer extends EventEmitter {
    caches: Map<string, Cache.Syncable<Docs.Base>>;
    constructor();
    sync(eventName: string, filter: EventFilter, contract: BaseContract, cache: Cache.Syncable<Docs.Base>, options?: Options.Sync): Promise<void>;
    loadCache<D extends Docs.Base, C extends Cache.Syncable<D> = Cache.Syncable<D>>(name: string): C;
    listenForEvents(name: string, contract: BaseContract, filter: EventFilter, cache: Cache.Syncable<Docs.Base>): void;
    clearListenerByIndex(contract: BaseContract, event: EventFilter, listenerIndex?: number): void;
    protected _populateSyncOptions(options: Options.Sync): Promise<DeepRequired<Options.Sync>>;
}
export declare function syncErrorHandler(err: Error, numResolvedPromises: number, callbackIndex: number, orderIndex: number, ...args: any[]): void;
export {};
//# sourceMappingURL=index.d.ts.map